#include <stdio.h>
#include <stdlib.h>
#include <tickLib.h>
#include <menuFtype.h>
#include "drvGtr.h"

/*************************************************************************
* Copyright (c) 2002 The University of Chicago, as Operator of Argonne
* National Laboratory, and the Regents of the University of California, as
* Operator of Los Alamos National Laboratory. EPICS BASE Versions 3.13.7
* and higher are distributed subject to a Software License Agreement found
* in file LICENSE that is included with this distribution.
*************************************************************************/

int timeReadMemoryGtr(int card, int nwords, int ntimes)
{
    unsigned long stime,etime;
    double totalTime,perAdcValue,perTime;
    int ind;
    gtrPvt gtrpvt;
    gtrops *pgtrops;
    int nchannels;
    gtrchannel *pachannel;
    gtrchannel **papchannel;

    
    gtrpvt = gtrFind(card,&pgtrops);
    if(!gtrpvt) {
        printf("gtrFind failed\n");
        return(0);
    }
    nchannels = (*pgtrops->numberChannels)(gtrpvt);
    pachannel = calloc(nchannels,sizeof(gtrchannel));
    papchannel = calloc(nchannels,sizeof(gtrchannel *));
    for(ind=0;ind<nchannels;ind++) {
       papchannel[ind] = &pachannel[ind];
       papchannel[ind]->len = nwords;
       papchannel[ind]->pdata = calloc(nwords,sizeof(int16));
       papchannel[ind]->ftvl = menuFtypeSHORT;
    }
    stime=tickGet();
    for(ind=0;ind<ntimes; ind++) {
        (*pgtrops->readMemory)(gtrpvt,papchannel);
    }
    etime=tickGet();
    totalTime=(etime-stime)/(double)sysClkRateGet();
    perTime = totalTime/ntimes;
    perAdcValue = perTime/(nwords*nchannels);
    printf( "nwords %d ntimes %d totalTime %f perTime %f perword %e\n",
                                nwords,ntimes,totalTime,perTime,perAdcValue);
    for(ind=0;ind<nchannels;ind++) {
       free(papchannel[ind]->pdata);
    }
    free(pachannel);
    return(0);
}

int timeReadRawMemoryGtr(int card, int nwords, int ntimes)
{
    unsigned long stime,etime;
    double totalTime,perAdcValue,perTime;
    int ind;
    gtrPvt gtrpvt;
    gtrops *pgtrops;
    int nchannels;
    gtrchannel *pachannel;
    gtrchannel **papchannel;

    
    gtrpvt = gtrFind(card,&pgtrops);
    if(!gtrpvt) {
        printf("gtrFind failed\n");
        return(0);
    }
    nchannels = (*pgtrops->numberRawChannels)(gtrpvt);
    pachannel = calloc(nchannels,sizeof(gtrchannel));
    papchannel = calloc(nchannels,sizeof(gtrchannel *));
    for(ind=0;ind<nchannels;ind++) {
       papchannel[ind] = &pachannel[ind];
       papchannel[ind]->len = nwords;
       papchannel[ind]->pdata = calloc(nwords,sizeof(long));
       papchannel[ind]->ftvl = menuFtypeLONG;
    }
    stime=tickGet();
    for(ind=0;ind<ntimes; ind++) {
        (*pgtrops->readRawMemory)(gtrpvt,papchannel);
    }
    etime=tickGet();
    totalTime=(etime-stime)/(double)sysClkRateGet();
    perTime = totalTime/ntimes;
    perAdcValue = perTime/(nwords*nchannels);
    printf( "nwords %d ntimes %d totalTime %f perTime %f perword %e\n",
                                nwords,ntimes,totalTime,perTime,perAdcValue);
    for(ind=0;ind<nchannels;ind++) {
       free(papchannel[ind]->pdata);
    }
    free(pachannel);
    return(0);
}
